<?php
//
// Copyright (c) 2008-2015 Nagios Enterprises, LLC. All rights reserved.
//
// $Id: dashlets.php 3793 2015-09-24 21:54:33Z jomann $

define("SKIPDASHLETS", 1);

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables and check pre-reqs
grab_request_vars();
check_prereqs();
check_authentication(false);

// Only admins can access this page
if (is_admin() == false) {
    echo $lstr['NotAuthorizedErrorText'];
    exit();
}

route_request();

function route_request()
{
    global $request;

    if (isset($request["download"]))
        do_download();
    else if (isset($request["upload"]))
        do_upload();
    else if (isset($request["delete"]))
        do_delete();
    else
        show_dashlets();

    exit;
}


function show_dashlets($error = false, $msg = "")
{
    global $lstr;
    global $dashlets;

    $dashlets = array();
    reset($dashlets);
    $p = dirname(__FILE__) . "/../includes/dashlets/";
    $subdirs = scandir($p);
    foreach ($subdirs as $sd) {
        if ($sd == "." || $sd == "..")
            continue;
        $d = $p . $sd;
        if (is_dir($d)) {
            $cf = $d . "/$sd.inc.php";
            if (file_exists($cf)) {
                $dashlets_copy = $dashlets;
                include_once($cf);
                $non_system_dashlets[basename($d)] = array_diff_assoc($dashlets, $dashlets_copy);
            }
        }
    }

    do_page_start(array("page_title" => _('Manage Dashlets')), true);
?>

    <h1><?php echo _('Manage Dashlets'); ?></h1>

    <?php display_message($error, false, $msg); ?>

    <p>
        <?php echo _('Manage the dashlets that are installed on this system and available to users. Need a custom dashlet created for your organization?'); ?> <a href="https://www.nagios.com/contact"><?php echo _('Contact us'); ?></a> <?php echo _('for pricing information.'); ?><br><?php echo _("You can find additional dashlets for Nagios XI at the"); ?> <a href='https://exchange.nagios.org/directory/Addons/Dashlets' target='_blank'><?php echo _('Nagios Exchange'); ?></a>.
    </p>

    <div class="well">
        <form enctype="multipart/form-data" action="dashlets.php" method="post" style="margin: 0;">
            <?php echo get_nagios_session_protector(); ?>
            <input type="hidden" name="upload" value="1">
            <input type="hidden" name="MAX_FILE_SIZE" value="1000000">

            <div class="fl" style="height: 29px; line-height: 29px; margin-right: 10px; font-weight: bold; color: #666;"><?php echo _('Upload a Dashlet'); ?></div>
            <div class="fl" style="margin-right: 10px;">
                <div class="input-group" style="width: 240px;">
                    <span class="input-group-btn">
                        <span class="btn btn-sm btn-default btn-file">
                            <?php echo _('Browse'); ?>&hellip; <input type="file" name="uploadedfile">
                        </span>
                    </span>
                    <input type="text" class="form-control" style="width: 200px;" readonly>
                </div>
            </div>
            <div class="fl">
                <button type="submit" class="btn btn-sm btn-primary"><?php echo _('Upload &amp; Install'); ?></button>
            </div>

            <div class="fr">
                <a href="https://exchange.nagios.org/directory/Addons/Dashlets" class="btn btn-sm btn-default"><?php echo _('More Dashlets'); ?> <i class="fa fa-external-link r"></i></a>
            </div>

            <div class="clear"></div>
        </form>
    </div>

    <?php
    foreach ($non_system_dashlets as $d => $non_dashlets) {
        $dashlet_dir = $d;
        foreach ($non_dashlets as $name => $darray) {
            show_dashlet($dashlet_dir, $name, $darray);
        }
    }
    ?>
    <div class="clear"></div>

<?php
    do_page_end(true);
    exit();
}


/**
 * @param $dashlet_dir
 * @param $dashlet_name
 * @param $darray
 * @param $x
 */
function show_dashlet($dashlet_dir, $dashlet_name, $darray)
{
    global $lstr;
    if (!array_key_exists('preview_class', $darray)) { return; }

    echo "<div class='fl md-box'>";
    display_dashlet_preview($dashlet_name, $darray);
    echo "<div class='md-buttons'><a href='?download=" . $dashlet_dir . "'><img src='" . theme_image("package_go.png") . "' class='tt-bind' alt='" . $lstr['DownloadAlt'] . "' title='" . $lstr['DownloadAlt'] . "'></a> ";
    echo "<a href='?delete=" . $dashlet_dir . "&nsp=" . get_nagios_session_protector_id() . "'><img src='" . theme_image("cross.png") . "' class='tt-bind' alt='" . $lstr['DeleteAlt'] . "' title='" . $lstr['DeleteAlt'] . "'></a></div>";
    echo "<div class='clear'></div></div>";
}

function do_download()
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_dashlets(true, $lstr['DemoModeChangeError']);

    $dashlet_dir = grab_request_var("download");
    if (have_value($dashlet_dir) == false)
        show_dashlets();

    // clean the name
    $dashlet_dir = str_replace("..", "", $dashlet_dir);
    $dashlet_dir = str_replace("/", "", $dashlet_dir);
    $dashlet_dir = str_replace("\\", "", $dashlet_dir);

    $id = submit_command(COMMAND_PACKAGE_DASHLET, $dashlet_dir);
    if ($id <= 0)
        show_dashlets(true, $lstr['ErrorSubmittingCommandText']);
    else {
        for ($x = 0; $x < 40; $x++) {
            $status_code = -1;
            $result_code = -1;
            $args = array(
                "cmd" => "getcommands",
                "command_id" => $id,
            );
            $xml = get_backend_xml_data($args);
            if ($xml) {
                if ($xml->command[0]) {
                    $status_code = intval($xml->command[0]->status_code);
                    $result_code = intval($xml->command[0]->result_code);
                }
            }
            if ($status_code == 2) {
                if ($result_code == 0) {

                    // wizard was packaged, send it to user
                    $dir = get_tmp_dir();
                    $thefile = $dir . "/dashlet-" . $dashlet_dir . ".zip";

                    //chdir($dir);

                    $mime_type = "";
                    header('Content-type: ' . $mime_type);
                    header("Content-length: " . filesize($thefile));
                    header('Content-Disposition: attachment; filename="' . basename($thefile) . '"');
                    readfile($thefile);
                } else
                    show_dashlets(true, $lstr['DashletPackagingTimedOutText']);
                exit();
            }
            usleep(500000);
        }
    }

    exit();
}


function do_upload()
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_dashlets(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    //print_r($request);
    //exit();

    $target_path = get_tmp_dir() . "/";
    $dashlet_file = basename($_FILES['uploadedfile']['name']);
    $target_path .= "dashlet-" . $dashlet_file;

    // log it
    send_to_audit_log("User installed dashlet '" . $dashlet_file . "'", AUDITLOGTYPE_CHANGE);

    //echo "TEMP NAME: ".$_FILES['uploadedfile']['tmp_name']."<BR>\n";
    //echo "TARGET: ".$target_path."<BR>\n";

    if (move_uploaded_file($_FILES['uploadedfile']['tmp_name'], $target_path)) {

        // fix perms
        chmod($target_path, 0550);
        chgrp($target_path, "nagios");

        $id = submit_command(COMMAND_INSTALL_DASHLET, $dashlet_file);
        if ($id <= 0)
            show_dashlets(true, $lstr['ErrorSubmittingCommandText']);
        else {
            for ($x = 0; $x < 20; $x++) {
                $status_code = -1;
                $result_code = -1;
                $args = array(
                    "cmd" => "getcommands",
                    "command_id" => $id,
                );
                $xml = get_backend_xml_data($args);
                if ($xml) {
                    if ($xml->command[0]) {
                        $status_code = intval($xml->command[0]->status_code);
                        $result_code = intval($xml->command[0]->result_code);
                        $result_text = strval($xml->command[0]->result);
                    }
                }
                if ($status_code == 2) {
                    if ($result_code == 0)
                        show_dashlets(false, $lstr['DashletInstalledText']);
                    else {
                        $emsg = "";
                        if ($result_text != "")
                            $emsg .= " " . $result_text . "";
                        show_dashlets(true, $lstr['DashletInstallFailedText'] . $emsg);
                        //show_dashlets(true,$lstr['DashletInstallFailedText']);
                    }
                    exit();
                }
                usleep(500000);
            }
        }
        show_dashlets(false, $lstr['DashletScheduledForInstallationText']);
    } else {
        // error
        show_dashlets(true, $lstr['DashletUploadFailedText']);
    }

    exit();
}

function do_delete()
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_dashlets(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    $dir = grab_request_var("delete", "");

    // clean the filename
    $dir = str_replace("..", "", $dir);
    $dir = str_replace("/", "", $dir);
    $dir = str_replace("\\", "", $dir);

    if ($dir == "")
        show_dashlets();

    // log it
    send_to_audit_log("User deleted dashlet '" . $dir . "'", AUDITLOGTYPE_DELETE);

    $id = submit_command(COMMAND_DELETE_DASHLET, $dir);
    if ($id <= 0)
        show_dashlets(true, $lstr['ErrorSubmittingCommandText']);
    else {
        for ($x = 0; $x < 14; $x++) {
            $status_code = -1;
            $args = array(
                "cmd" => "getcommands",
                "command_id" => $id,
            );
            $xml = get_backend_xml_data($args);
            if ($xml) {
                if ($xml->command[0]) {
                    $status_code = intval($xml->command[0]->status_code);
                }
            }
            if ($status_code == 2) {
                show_dashlets(false, $lstr['DashletDeletedText']);
                exit();
            }
            usleep(500000);
        }
    }
    show_dashlets(false, $lstr['DashletScheduledForDeletionText']);
    exit();
}

?>